// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'package:nim_core/nim_core.dart';
import 'package:hawk/hawk.dart';

import 'method_name_value.dart';

class HandlePinCase extends HandleBaseCase {
  HandlePinCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    var inputParams = Map<String, dynamic>();
    if (params != null && params!.length > 0) {
      inputParams = params![0] as Map<String, dynamic>;
    }

    switch (methodName) {
      case addMessagePin:
        {
          NIMResult<void> result = await NimCore.instance.messageService
              .addMessagePin(NIMMessage.fromMap(inputParams['message']),
                  inputParams['ext']);
          handleCaseResult = ResultBean(
              code: result.code, data: result.toMap(), message: methodName);
          break;
        }
      case updateMessagePin:
        {
          final result = await NimCore.instance.messageService.updateMessagePin(
              NIMMessage.fromMap(inputParams['message']), inputParams['ext']);
          handleCaseResult = ResultBean(
              code: result.code, message: methodName, data: result.toMap());
          break;
        }
      case removeMessagePin:
        {
          final result = await NimCore.instance.messageService.removeMessagePin(
              NIMMessage.fromMap(inputParams['message']), inputParams['ext']);
          handleCaseResult = ResultBean(
              code: result.code, message: methodName, data: result.toMap());
          break;
        }
      case queryMessagePinForSession:
        {
          final result = await NimCore.instance.messageService
              .queryMessagePinForSession(
                  inputParams['sessionId'],
                  NIMSessionTypeConverter()
                      .fromValue(inputParams['sessionType'] as String));
          handleCaseResult = ResultBean(
              code: result.code, message: methodName, data: result.toMap());
          break;
        }
    }
    return handleCaseResult;
  }
}
